﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Web;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;

namespace SampleWebApp
{
    [Serializable]
    public class CustomClusterValueStyle : Style
    {
        private string columnName;
        private Dictionary<string, ValueItem> valueItems = new Dictionary<string, ValueItem>();

        public CustomClusterValueStyle()
            : this(string.Empty, new Dictionary<string, ValueItem>())
        {
        }

        public CustomClusterValueStyle(string columnName, Dictionary<string, ValueItem> valueItems)
        {
            this.columnName = columnName;
            this.valueItems = valueItems;
        }

        public string ColumnName
        {
            get { return columnName; }
            set { columnName = value; }
        }

        // We use a dictionary now instead of a collection, to make lookups faster.
        public Dictionary<string, ValueItem> ValueItems
        {
            get { return valueItems; }
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            try
            {
                double scale = ExtentHelper.GetScale(canvas.CurrentWorldExtent, canvas.Width, canvas.MapUnit);
                //if (scale2 != scale)
                //{
                //    scale2 = scale;
                MapSuiteTileMatrix mapSuiteTileMatrix = new MapSuiteTileMatrix(scale, 10, 10, canvas.MapUnit);
                IEnumerable<TileMatrixCell> tileMatricCells = mapSuiteTileMatrix.GetContainedCells(canvas.CurrentWorldExtent);
                Dictionary<string, string> unusedFeatures = new Dictionary<string, string>();
                foreach (Feature feature in features)
                    unusedFeatures.Add(feature.Id, feature.Id);
                Collection<Feature> featuresFinal = new Collection<Feature>();
                Feature[] tmpFeatures = null;
                ValueItem valueItem = null;
                MultipointShape multiPointShape = new MultipointShape();
                List<Feature> featuresPointsCorrected = null;
                foreach (TileMatrixCell cell in tileMatricCells)
                {
                    if (featuresFinal.Count > 0)
                        featuresFinal.Clear();
                    if (multiPointShape.Points.Count > 0)
                        multiPointShape.Points.Clear();
                    featuresPointsCorrected = features.AsEnumerable().Where(ft => (unusedFeatures.ContainsKey(ft.Id) && cell.BoundingBox.Contains(ft.GetBoundingBox()))).ToList();
                    if (featuresPointsCorrected.Count > 0)
                    {
                        featuresPointsCorrected.ForEach(ft1 => multiPointShape.Points.Add(ft1.GetBoundingBox().GetCenterPoint()));
                        foreach (Feature feature in featuresPointsCorrected)
                        {
                            unusedFeatures.Remove(feature.Id);
                             valueItem = null;
                            if (valueItems.ContainsKey(feature.ColumnValues[columnName].Trim()))
                                valueItem = valueItems[feature.ColumnValues[columnName].Trim()];
                            if (valueItem != null)
                            {
                                tmpFeatures = new Feature[1] { new Feature(multiPointShape.GetCenterPoint()) };
                                if (valueItem.CustomStyles.Count == 0)
                                {
                                    valueItem.DefaultPointStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers); break;
                                }
                                else
                                {
                                    foreach (Style style in valueItem.CustomStyles)
                                        style.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers);
                                }
                            }
                        }
                    }
                }
                //}
            }
            catch (Exception)
            { }
        }

        protected override Collection<string> GetRequiredColumnNamesCore()
        {
            Collection<string> requiredFieldNames = new Collection<string>();

            requiredFieldNames.Add(columnName);

            foreach (ValueItem valueItem in valueItems.Values)
            {
                foreach (Style style in valueItem.CustomStyles)
                {
                    Collection<string> tmpCollection = style.GetRequiredColumnNames();

                    foreach (string name in tmpCollection)
                    {
                        if (!requiredFieldNames.Contains(name))
                        {
                            requiredFieldNames.Add(name);
                        }
                    }
                }
                Collection<string> fieldsInTextStyle = valueItem.DefaultTextStyle.GetRequiredColumnNames();

                foreach (string fieldName in fieldsInTextStyle)
                {
                    if (!requiredFieldNames.Contains(fieldName))
                    {
                        requiredFieldNames.Add(fieldName);
                    }
                }
            }
            return requiredFieldNames;
        }

    }
}