﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WebForms;
using P4 = Proj4Net;
namespace SampleWebApp
{
    public partial class Sample_WorldWrapMapClick : System.Web.UI.Page
    {
        Proj4Projection _proj = new Proj4Projection();
        static RectangleShape _rectShape_WrapExtent = new RectangleShape(-180, 90, 180, -90);
        protected void Page_Load(object sender, EventArgs e)
        {

            try
            {
                LoadDefaultMap();
                PlotRoute();
                PlotAircraft();

                if ((wfMap.CustomOverlays.Contains("RouteOverlay")) && ((LayerOverlay)wfMap.CustomOverlays["RouteOverlay"]).Layers.Contains("RouteLayer"))
                {
                    InMemoryFeatureLayer imRouteLayer = (InMemoryFeatureLayer)((LayerOverlay)wfMap.CustomOverlays["RouteOverlay"]).Layers["RouteLayer"];
                    imRouteLayer.Open();
                    wfMap.CurrentExtent = imRouteLayer.GetBoundingBox();
                    imRouteLayer.Close();
                }
                else
                    wfMap.CurrentExtent = _proj.ConvertToExternalProjection(new RectangleShape(-130, 60, 130, -60));
            }
            catch (Exception)
            {
                throw new Exception();
            }
        }

        private void LoadDefaultMap()
        {
            try
            {
                if (wfMap.CustomOverlays.Count > 0)
                    wfMap.CustomOverlays.Clear();

                _proj.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
                _proj.ExternalProjectionParametersString = Proj4Projection.GetEsriParametersString(54004);

                _proj.Open();
                wfMap.MapUnit = _proj.GetExternalGeographyUnit();
                wfMap.MapBackground = new GeoSolidBrush(GeoColors.AliceBlue);

                ShapeFileFeatureLayer shpWorldLayer = new ShapeFileFeatureLayer(@"F:\Riyaz\RnD\Web\SampleWebApp\SampleWebApp\MapData\Countries02.shp", @"F:\Riyaz\RnD\Web\SampleWebApp\SampleWebApp\MapData\Countries02.idx", GeoFileReadWriteMode.Read);
                shpWorldLayer.FeatureSource.Projection = _proj;
                shpWorldLayer.WrappingMode = WrappingMode.WrapDateline;
                shpWorldLayer.WrappingExtent = _proj.ConvertToExternalProjection(_rectShape_WrapExtent);
                shpWorldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColors.LightGreen), new GeoSolidBrush(GeoColors.Transparent));
                shpWorldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay loWorld = new LayerOverlay("WorldOverlay", true, TileType.SingleTile);
                loWorld.Layers.Add("WorldLayer", shpWorldLayer);
                wfMap.CustomOverlays.Add(loWorld);

            }
            catch (Exception)
            {
                throw new Exception();
            }
        }

        private void PlotAircraft()
        {
            try
            {
                InMemoryFeatureLayer imAircraftLayer = new InMemoryFeatureLayer();
                imAircraftLayer.FeatureSource.Projection = _proj;
                imAircraftLayer.Open();
                imAircraftLayer.Columns.Add(new FeatureSourceColumn("FlightID"));
                imAircraftLayer.WrappingMode = WrappingMode.WrapDateline;
                imAircraftLayer.WrappingExtent = _proj.ConvertToExternalProjection(_rectShape_WrapExtent);

                Feature ft = new Feature(new PointShape(15, 10));
                ft.ColumnValues.Add("FlightID", "101");
                imAircraftLayer.InternalFeatures.Add(ft);

                imAircraftLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColors.Red), 10);
                imAircraftLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay loAircraft = new LayerOverlay("AircraftOverlay", false, TileType.SingleTile);

                loAircraft.Layers.Add("AircraftLayer", imAircraftLayer);
                wfMap.CustomOverlays.Add(loAircraft);
            }
            catch (Exception)
            {
                throw new Exception();
            }
        }

        private void PlotRoute()
        {
            try
            {
                InMemoryFeatureLayer imRouteLayer = new InMemoryFeatureLayer();
                imRouteLayer.FeatureSource.Projection = _proj;
                imRouteLayer.Open();
                imRouteLayer.Columns.Add(new FeatureSourceColumn("FlightID"));
                imRouteLayer.Columns.Add(new FeatureSourceColumn("Type"));
                imRouteLayer.WrappingMode = WrappingMode.WrapDateline;
                imRouteLayer.WrappingExtent = _proj.ConvertToExternalProjection(_rectShape_WrapExtent);

                #region ValueStyle with ClusterPointStyle
                Feature ft = new Feature(new PointShape(10, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "1");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(11, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "1");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(12, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "2");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(13, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "2");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(14, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "1");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(15, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "1");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(16, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "1");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(17, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "2");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(18, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "1");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(19, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "1");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new PointShape(20, 10));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "2");
                imRouteLayer.InternalFeatures.Add(ft);

                ft = new Feature(new LineShape("LINESTRING(10 10,11 10,12 10,13 10,14 10,15 10,16 10,17 10,18 10,19 10,20 10)"));
                ft.ColumnValues.Add("FlightID", "101");
                ft.ColumnValues.Add("Type", "");
                imRouteLayer.InternalFeatures.Add(ft);

                ValueStyle vs = new ValueStyle();
                vs.ColumnName = "Type";

                ValueItem vi = new ValueItem();
                vi.Value = "1";
                vi.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColors.Blue), 3);
                vs.ValueItems.Add(vi);

                vi = new ValueItem();
                vi.Value = "2";
                vi.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColors.Green), 3);
                vs.ValueItems.Add(vi);

                imRouteLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(vs);
                imRouteLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(new LineStyle(new GeoPen(GeoColors.Orange)))
                    ;
                imRouteLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                #endregion

                LayerOverlay loRoute = new LayerOverlay("RouteOverlay", false, TileType.SingleTile);
                loRoute.Layers.Add("RouteLayer", imRouteLayer);
                wfMap.CustomOverlays.Add(loRoute);
            }
            catch (Exception)
            {
                throw new Exception();
            }
        }

        static bool isTrue = true;
        protected void wfMap_Click(object sender, MapClickedEventArgs e)
        {
            try
            {
                if (wfMap.CustomOverlays.Contains("AircraftOverlay") && ((LayerOverlay)wfMap.CustomOverlays["AircraftOverlay"]).Layers.Contains("AircraftLayer"))
                {
                    InMemoryFeatureLayer imAircraftLayer = (InMemoryFeatureLayer)((LayerOverlay)wfMap.CustomOverlays["AircraftOverlay"]).Layers["AircraftLayer"];
                    if (!imAircraftLayer.IsOpen)
                        imAircraftLayer.Open();
                    if (!imAircraftLayer.FeatureSource.IsOpen)
                        imAircraftLayer.FeatureSource.Open();
                    #region Old
                    //PointShape ptPosition = null;
                    //if (e.Position.X >= -180 && e.Position.X <= 180)
                    //    ptPosition = e.Position;
                    //else
                    //{
                    //    //var Xnew = e.Position.X + 360 * count(-180 < Xnew < 180)
                    //    if (e.Position.X > 180)
                    //        ptPosition = new PointShape((e.Position.X % 360), e.Position.Y);
                    //    else
                    //    {
                    //        var x1 = 360 -Math.Abs(e.Position.X);
                    //        ptPosition = new PointShape(x1, e.Position.Y);
                    //    }
                    //}
                    #endregion
                    if (!_proj.IsOpen)
                        _proj.Open();
                    //UnmanagedProj4Projection pr = new UnmanagedProj4Projection();
                    //pr.ExternalProjectionParametersString = UnmanagedProj4Projection.GetEsriParametersString(54004);
                    //pr.InternalProjectionParametersString = UnmanagedProj4Projection.GetEpsgParametersString(4326);
                    //_proj.ExternalProjectionParametersString = "+proj=merc +lon_0=0 +k=1 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs";
                    //_proj.InternalProjectionParametersString = "+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext  +no_defs";
                    //pr.Open();
                    //var x = _proj.ConvertToInternalProjection(e.Position.X, e.Position.Y);

                    PointShape ptPosition = (PointShape)_proj.ConvertToInternalProjection(e.Position);
                    if (ptPosition.X <= -180 && ptPosition.X >= 180)
                    {
                        if (ptPosition.X > 180)
                            ptPosition = new PointShape((ptPosition.X % 360), ptPosition.Y);
                        else
                            ptPosition = new PointShape((360 - Math.Abs(ptPosition.X)), ptPosition.Y);
                    }
                    ptPosition = (PointShape)_proj.ConvertToExternalProjection(ptPosition);
                    Collection<Feature> fts_SelectedAircrafts = imAircraftLayer.FeatureSource.GetFeaturesWithinDistanceOf(ptPosition, wfMap.MapUnit, DistanceUnit.NauticalMile, 500, ReturningColumnsType.AllColumns);
                    if (fts_SelectedAircrafts.Count > 0 && (wfMap.CustomOverlays.Contains("RouteOverlay") && ((LayerOverlay)wfMap.CustomOverlays["RouteOverlay"]).Layers.Contains("RouteLayer")))
                    {
                        if (isTrue)
                            isTrue = false;
                        else
                            isTrue = true;
                        ((InMemoryFeatureLayer)((LayerOverlay)wfMap.CustomOverlays["RouteOverlay"]).Layers["RouteLayer"]).IsVisible = isTrue;
                        ScriptManager.RegisterStartupScript(this, this.GetType(), "loadingRouteOverlay", "updateOverlay('RouteOverlay');", true);
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

    }
}